"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ono_1 = require("ono");
const clone_1 = require("./clone");
/**
 * Returns a cloneable copy of the given error.
 * @internal
 */
function cloneError(error) {
    return clone_1.clone(error);
}
exports.cloneError = cloneError;
/**
 * Creates an `Error` object from an `ErrorClone`.
 * @internal
 */
function createError(error) {
    if (!error || typeof error !== "object" || error instanceof Error) {
        // It's not an ErrorClone, so just return it as-is
        return error;
    }
    else if (error.name in builtInErrorTypes) {
        // Convert the ErrorClone to the corresponding Error type
        let errorType = builtInErrorTypes[error.name];
        return errorType(error);
    }
    else {
        // Convert the ErrorClone to an Error instance
        return ono_1.ono(error);
    }
}
exports.createError = createError;
/**
 * Function that coerce an `ErrorClone` to one of the built-in JavaScript error types
 */
const builtInErrorTypes = {
    [EvalError.name]: ono_1.ono.eval,
    [RangeError.name]: ono_1.ono.range,
    [ReferenceError.name]: ono_1.ono.reference,
    [SyntaxError.name]: ono_1.ono.syntax,
    [TypeError.name]: ono_1.ono.type,
    [URIError.name]: ono_1.ono.uri,
};
//# sourceMappingURL=clone-error.js.map