"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ono_1 = require("ono");
const clone_helpers_1 = require("./clone-helpers");
const defaultDepth = 5;
/**
 * Returns a cloneable version of the given value. If the value is already cloneable, then it is
 * returned as-is. Otherwise, it is coerced into a cloneable value, which may be lossy.
 * @internal
 */
function clone(value, depth = defaultDepth) {
    if (clone_helpers_1.isCloneable(value, depth)) {
        // This value is natively cloneable. So just return it as-is.
        return value;
    }
    if (value instanceof Error) {
        return clone(ono_1.Ono.toJSON(value));
    }
    if (Array.isArray(value)) {
        return value.map((item) => clone(item, depth));
    }
    if (value instanceof Map) {
        let copy = new Map();
        for (let [k, v] of value.entries()) {
            copy.set(clone(k, depth), clone(v, depth));
        }
        return copy;
    }
    if (value instanceof Set) {
        let copy = new Set();
        for (let v of value.values()) {
            copy.add(clone(v, depth));
        }
        return copy;
    }
    if (typeof value === "object") {
        let copy = {};
        for (let key of clone_helpers_1.getPropertyNames(value)) {
            let prop = value[key];
            copy[key] = depth > 0 ? clone(prop, depth - 1) : undefined;
        }
        return copy;
    }
}
exports.clone = clone;
//# sourceMappingURL=clone.js.map