"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ono_1 = require("ono");
const resolveFrom = require("resolve-from");
const resolveGlobal = require("resolve-global");
const worker_threads_1 = require("worker_threads");
/**
 * Imports the specified JavaScript module, either from the current path,
 * the local "node_modules" folder, or a globally-installed NPM package.
 *
 * @param moduleId - The name or path of the module to import
 * @param [cwd] - The local directory to start searching for the module
 * @internal
 */
async function importModule(moduleId, cwd) {
    let filename = resolveFrom.silent(cwd || __dirname, moduleId) || resolveGlobal.silent(moduleId);
    if (!filename) {
        throw ono_1.ono({ workerId: worker_threads_1.threadId, moduleId }, `Cannot find module "${moduleId}" in the local path or as a globally-installed package.`);
    }
    return Promise.resolve().then(() => require(filename));
}
exports.importModule = importModule;
//# sourceMappingURL=import-module.js.map